package ACL2::Orden::Defun;
use base ACL2::Orden;

=head1 NAME

  ACL2::Orden::Defun - funciE<oacute>n Lisp aplicativa con parada demostrada

=head1 FUNCTIONS

=cut

#
#    Orden Defun: función Lisp aplicativa con parada demostrada.
#    Copyright (C) 2006-2008 Antonio García Domínguez
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use warnings;
use utf8;

use ACL2::Resultado;
use ACL2::Tags;
use ACL2::Localizacion;
use ACL2::Proceso;

my $lh = ACL2::Localizacion->get_handle();

=head2 get_identificador_evento

Uso: ME<eacute>todo pE<uacute>blico.

Responsabilidades: Devuelve el identificador del evento que pasarE<aacute> a
entrar en el mundo lE<oacute>gico de ACL2.

ParE<aacute>metros: Ninguno.

=cut

sub get_identificador_evento
{
    my ($this) = @_;

    return $this->{paquete} . "::" . $this->{nombre};
}

sub protegido_inicializar_sexpr
{
    my ($this) = @_;

    # Especificamos los atributos existentes, con valores por defecto
    $this->{nombre}  = $this->get_elems_formula->[1]; $this->{nombre} =~ tr/a-z/A-Z/;
    $this->{args}    = $this->get_elems_formula->[2];
    $this->{formula} = ${$this->get_lisp};
    $this->{exito}   = 1;
}

sub protegido_inicializar_salida
{
    my ($this) = @_;

    # Listamos otras propiedades que se establecen en los métodos privados
    $this->{adm_relacion}     = undef;
    $this->{adm_medida}       = undef;
    $this->{demostracion}     = undef;
    $this->{tipo_prefijo}     = undef;
    $this->{tipo_salida}      = undef;
    $this->{tipo_restriccion} = undef;
    $this->{tipo_reglas}      = undef;

    $this->_procesar_parada;
    $this->_procesar_tipo if $this->{exito} == 1;
}

sub protegido_cabecera
{
    my ($orden, $writer) = @_;
    $writer->startTag(ACL2::Tags::DEFUN_TAG,
		      "name" => $orden->{nombre},
		      "args" => $orden->{args},
		      "formula" => $orden->{formula},
		      "package" => $orden->get_paquete);
}

sub protegido_cuerpo
{
    my ($orden, $writer) = @_;

    # Demostración de la parada
    if (defined($orden->{adm_relacion})) {
	$writer->emptyTag(ACL2::Tags::ADMISSION_TAG, 
			  "relation" => $orden->{adm_relacion},
			  "measure" => $orden->{adm_medida});
    }

    # Demostración, si la hay
    if (defined($orden->{demostracion})) 
    {
	ACL2::Orden->salida_parrafos($writer, $orden->{dem_prefijo});
	$orden->{demostracion}->escribir_xml($writer);
    }
    
    # Restricción sobre el tipo de la función    
    if ($orden->{exito}) 
    {
	  my $endTag;
	  ACL2::Orden->salida_parrafos($writer, $orden->{tipo_prefijo});
	  
	  if (defined($orden->{tipo_restriccion})) {
	      $writer->startTag($endTag = ACL2::Tags::TIPO_TAG, 
				"formula" => $orden->{tipo_restriccion});
	  } else {
	      $writer->startTag($endTag = ACL2::Tags::NOTIPO_TAG);
	  }
	  
	  ACL2::Proceso->volcar_reglas($writer, $orden->{tipo_reglas});
	  
	  ACL2::Orden->salida_parrafos($writer, $orden->{tipo_salida});
	  $writer->endTag($endTag);
      }
}



#
# MÉTODOS PRIVADOS
#



=head2 _procesar_parada

Uso: ME<eacute>todo privado.

Responsabilidad: Extrae la informaciE<oacute>n acerca de la parada del texto.

ParE<aacute>metros: Ninguno.

Devuelve: Nada.

=cut

sub _procesar_parada
{
    caller eq __PACKAGE__ or die;
    my ($this) = @_;
    my $nombre = $this->{nombre};

    for (${$this->get_salida}) {
	/(?-x:The admission of \Q$nombre\E is trivial, using the relation (.*?) .* and )
	    (?-x:the measure (.*?)\.)/x 
	    && do 
	{
	    $this->{adm_relacion} = $1;
	    $this->{adm_medida}  = $2;
	    last;
	};

	/Since \Q$nombre\E is non-recursive/ && do 
	{
	    $this->{adm_relacion} = "_trivial_";
	    $this->{adm_medida} = "_trivial_";
	    last;
	};

	/(?-x:\s*For the admission of \Q$nombre\E we .*? relation (.*?) .*? and the measure (.*?)\.)
	    (?-x:.*The non-trivial part of the measure conjecture is[^\(]+(.*?)\.)/xs
	    && do 
	{
	    my ($relation, $measure, $measureFormula) = ($1,$2,$3);
	    my $textoDem = substr $_, $+[0];

	    $this->{adm_relacion} = $relation;
	    $this->{adm_medida}   = $measure;
	    $this->{demostracion} = new ACL2::Resultado($measureFormula, $textoDem);
	    $this->{exito} = $this->{demostracion}->demostrar;

	    # Retiramos la salida de la demostración para próximas invocaciones
	    $this->{dem_prefijo} = (substr $_, 0, $+[2]) . ".";
	    $this->set_salida(substr $_, $+[0] + $this->{demostracion}->get_long_dem);

	    last;	    
	};

	/^\s*$/ && do {
	    # Normally, a completely empty demonstration means that there was a fatal error
	    $this->{exito} = 0;
	    last;
	};

	die $lh->maketext("No se halló la relación o la medida");
    }
}


=head2 _procesar_tipo

Uso: ME<eacute>todo privado.

Responsabilidades: Procesa el texto referente al tipo que va a tener
la funciE<oacute>n.

ParE<aacute>metros: Ninguno.

Devuelve: Nada.

=cut

sub _procesar_tipo
{
    caller eq __PACKAGE__ or die;
    my ($orden) = @_;
    my $nombre  = $orden->{nombre};
    my $texto   = ${$orden->get_salida};

    for ($texto) {
	/(?-x:We observe that the type of \Q\U$nombre\E\E is described by the )
	    (?-x:theorem ([^\.]+)\.(?>\sWe used ([^\.]+)\.)?)/x 
	    && do 
	{
            $orden->{tipo_prefijo} = substr $texto, 0, $-[0]-1;
	    $orden->{tipo_restriccion} = $1;
	    $orden->{tipo_reglas}      = (defined($2) ? 
					  ACL2::Proceso->obtener_reglas($2) : 
					  []);
	    $orden->{tipo_salida}      = substr($texto,$-[0],$+[0] - $-[0]);
 	    last;
	};
	
	/(?-x:We could deduce no constraints on the type of (?:.*?)\.\s)
	    (?-x:(?>However, in normalizing the definition we used (.*?)\.)?)/oxs 
	    && do 
	{
            $orden->{tipo_prefijo} = substr $texto, 0, $-[0]-1;
	    $orden->{tipo_reglas}  = (defined($1) ? 
				      ACL2::Proceso->obtener_reglas($1) : 
				      []);
	    $orden->{tipo_salida}  = substr $texto, $-[0], $+[0]-$-[0];;
	    last;
	};

	/^\s*$/ && last;

	die $lh->maketext("Se esperaba el tipo de la función.");	
    }

}

1;
