package ACL2::Orden::Fabrica;
use ACL2::Orden;
use ACL2::AnalisisLisp;

#
#    Fábrica de Órdenes
#    Copyright (C) 2006-2008 Antonio García Domínguez
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use ACL2::Orden::DefinitionCategory;
use ACL2::Orden::FallbackCategory;

use strict;
use warnings;
use utf8;

use ACL2::Localizacion;
my $lh = ACL2::Localizacion->get_handle();

sub _parse_cmd
{
    my ($lisp, $rv_lisp_tree, $output, $pkg, $class) = @_;

    my $cmd = bless { str_lisp => \$lisp,
		      lisp   => $rv_lisp_tree,
		      salida => \$output,
		      paquete => $pkg}, $class;

    # Hacemos que se inicialice
    $cmd->inicializar;

    return $cmd;
}

=head2 crear

  Uso: Método público estático.
  Responsabilidades: 
    Crea el tipo correcto de Orden para una
    S-expresión y salida dadas.
  Parámetros:

=over 2

=item *

orden: S-expresión con la orden completa para ACL2.

=item *

salida: Salida completa de ACL2.

=item *

paquete: Paquete Lisp al que pertenece la fórmula.

=back

  Devuelve: 
    Descendiente de ACL2::Orden correspondiente, ya inicializado.
  Errores:
    Fallo fatal si la clase no existe.

=cut

sub crear
{
    my ($proto, $lisp, $salida, $paquete) = @_;

    # Change the contents of the theorem to uppercase, like ACL2 does
    $lisp = uc $lisp;

    # Parse the Lisp list into a tree of Perl references to lists and scalars
    my @arbolLisp = ACL2::AnalisisLisp->dividir_lista($lisp);
    # Remove non-alphanumeric characters from the command's name
    $arbolLisp[0] =~ s/\W//g;
    my $clase     = "ACL2::Orden::" . ucfirst lc $arbolLisp[0];

    # If a specific module exists for a command, use it
    # (ACL2::Orden::Defun for DEFUN, for instance). NOTE:
    # such modules must be "use"d in ACL2::Orden, or else
    # they won't be found.
    eval "require $clase" && do {
	# Found a specific module: use it
	return _parse_cmd($lisp, \@arbolLisp, $salida, $paquete, $clase);
    };

    # Otherwise, try to fall back to some of the generic command categories
    foreach my $category (("ACL2::Orden::DefinitionCategory",
			   "ACL2::Orden::FallbackCategory",)) {
	my $cmd = eval {
	    return _parse_cmd($lisp, \@arbolLisp, $salida, $paquete, $category);
	};
	return $cmd if defined($cmd);
    }

    # No specific module or fitting category was found: die with an error
    die $lh->maketext("No se encontró un módulo específico o categoría que concordara con [_1]", $clase);
}

1;

