package ACL2::Orden::FallbackCategory;
use base ACL2::Orden;

#
#   FallbackCategory: divide into paragraphs, S-exprs and summary
#   Copyright (C) 2006-2009 Antonio García Domínguez
#
#    This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use warnings;
use utf8;

use ACL2::Tags;

# Commands for which this sort of output suffices
my %KNOWN_CMDS = (
    "program" => ACL2::Tags::PROGRAM_TAG,
    "logic" => ACL2::Tags::LOGIC_TAG
);

sub protegido_cabecera
{
    my ($this, $writer) = @_;
    $writer->startTag($this->{tag},
		      "formula" => ${$this->get_lisp});
}

sub protegido_final
{
    my ($this, $writer) = @_;
    $writer->endTag($this->{tag});
}

sub protegido_inicializar_sexpr
{
    my ($this) = @_;

    my $cmd = lc $_[0]->get_elems_formula->[0];
    $cmd    =~ s/\W//g;

    if (exists $KNOWN_CMDS{$cmd})
    {
	$this->{tag} = $KNOWN_CMDS{$cmd};
    }
    else
    {
	$this->{tag} = ACL2::Tags::UNKNOWN_TAG;
    }
}

sub protegido_inicializar_salida {}

sub protegido_cuerpo
{
    my ($this, $writer) = @_;

    ACL2::Orden->salida_parrafos($writer, ${$this->get_salida});
}

1;
